@@ -26,6 +26,14 @@ PAUSED = 2
 DONE = 3
 STOPPED = 4
 ERROR = 5
+WARNING = 6
+
+ABORT = "Abort macro"
+SKIP = "Skip"
+
+MODE_ALL = 2
+MODE_EXTENDED = 1
+MODE_SIMPLE = 0
 
 # Attribute values
 VALUE = "Value"
@@ -44,6 +52,7 @@ OFFSET = "Offset"
 TOLERANCE = "Tolerance"
 SETPOINT = "Setpoint"
 PREVIOUS = "Previous"
+SAVE_CHANGE = "Save"
 ALL = "*"
 DISPLAY = "display"
 IMAGE = "Image"
@@ -393,6 +402,7 @@ ATTR_AVAILABLE_MEMORY = u"Available memory"
 ATTR_HOME_SWITCH_FLAG = u"Home switch flag"
 ATTR_HARDWARE_VERSION = u"Hardware version"
 ATTR_SOFTWARE_VERSION = u"Software version"
+ATTR_MANUFACTURE_DATE = u"Manufacture date"
 ATTR_APPLICATION_NUMBER = u"Application number"
 ATTR_APPLICATION_VERSION = u"Application version"
 ATTR_OPERATION_MODE = u"Operation mode"
@@ -421,4 +431,13 @@ ATTR_LOGFILE = u"Logfile"
 ATTR_HEADER = u"Header"
 ATTR_LAST_LOG_LINE = u"Last log line"
 ATTR_INTERVAL = u"Interval"
-ATTR_LOGGING = u"Logging"
\ No newline at end of file
+ATTR_LOGGING = u"Logging"
+ATTR_MEASURING_SCALE = u"Measuring scale"
+ATTR_DISTANCE = u"Distance"
+ATTR_MODES = u"Modes"
+ATTR_AUTO_STOP = u"Auto stop"
+ATTR_ABSOLUTE_POSITION = u"Absolute position"
+ATTR_WARNING = u"Warning"
+ATTR_VELOCITY_TIMEOUT = u"Velocity timeout"
+ATTR_SPEED_TABLE_ROW = u"Speed table row"
+ATTR_RESISTANCE = u"Resistance"
