@@ -313,7 +313,7 @@ class FileWorker(Worker, Process):
                 os.utime(dest_file, (float(file_mod_time.decode()), float(file_mod_time.decode())))
                 message = self.socket.recv_multipart()
                 if message[2] == MSG_ACK:
-                    self.socket.send_multipart([self.master_address, MSG_EMPTY, WORKER_DONE, task_id])
+                    self.socket.send_multipart([self.master_address, MSG_EMPTY, WORKER_DONE, task_id, source_file_path])
             else:
                 # Remove partial file
                 os.remove(dest_file)
@@ -380,7 +380,7 @@ class DeviceFileServer(Device, BalancedServer, Thread):
                     task_id = request[3].decode()
                     if self.get_value([task_id, ATTR_TASK_INSTRUCTION]) == "Sync files":
                         # If task is to sync file continue with next file
-                        if self._sync_next_file(task_id, request[0]):
+                        if self._sync_next_file(task_id, request[0], request[4].decode()):
                             continue
                     # Task is done
                     self.set_task_status(task_id, STATUS_IDLE)
@@ -442,7 +442,7 @@ class DeviceFileServer(Device, BalancedServer, Thread):
             # Unregister fronted socket from polling, no free worker available
             self.poller.unregister(self.socket_frontend)
 
-    def _sync_next_file(self, task_id, worker_id):
+    def _sync_next_file(self, task_id, worker_id, synced_file=None):
         """
         This is specific to sync request.
         When multiple files are requested, sync Task uses one worker to sync files one by one.
@@ -459,7 +459,7 @@ class DeviceFileServer(Device, BalancedServer, Thread):
         for drive, path in self.get_value([task_id, ATTR_REQUESTED_FILES]):
             input_dir, file_name = os.path.split(path)
             synced, local_path = self.get_local_path(drive, input_dir, file_name)
-            if synced:
+            if synced or path == synced_file:
                 synced_files[(drive, path)] = local_path
 
         self.set_value([task_id, ATTR_SYNCED_FILES], synced_files)
