@@ -1,5 +1,6 @@
 import re
 import time
+from math import ceil
 
 import numpy as np
 from bidict import frozenbidict
@@ -192,7 +193,7 @@ class DeviceKohzu(DeviceSocket):
             readout_buffer = self.error_description.get(int(error_code), u"Undefined error")
             self.handle_command_error(command, readout_buffer)
             s = re.search("[A-Z]+([0-9]+)", command)
-            command = RESPONSE_ERROR
+            command = RESPONSE_ERROR.decode()
             if s is not None and s != -1:
                 channel = int(s.group(1)) - 1
                 self.notify((channel, ATTR_LAST_ERROR), readout_buffer)
@@ -206,6 +207,7 @@ class DeviceKohzu(DeviceSocket):
             self.notify((channel, self.system_setting.inv[int(output_parts[1])]), int(output_parts[2]))
         elif output_parts[0][:3] == "RTB":
             channel = int(output_parts[0][3:]) - 1
+            self.notify((channel, ATTR_SPEED_TABLE_ROW), ", ".join(output_parts))
             self.notify((channel, ATTR_START_SPEED), int(output_parts[2]))
             self.notify((channel, ATTR_TOP_SPEED), int(output_parts[3]))
             self.notify((channel, ATTR_ACCELERATION_TIME), int(output_parts[4]))
@@ -319,7 +321,8 @@ class DeviceKohzuChannel(DeviceChannel):
                               set_function=lambda v: self.write_speed_table(acceleration_pattern=v))
         self.create_attribute(ATTR_READ_ENCODER, default_value=True, readonly=False, default_type=np.bool)
         self.create_attribute(ATTR_VELOCITY, default_value=0, default_type=np.int64, min_value=0, max_value=3e6,
-                              unit="nm/s", set_function=self.set_velocity)
+                              unit="nm/s", set_function=self.set_velocity, set_value_when_set_function=False)
+        self.create_attribute(ATTR_SPEED_TABLE_ROW, readonly=True)
 
     def handle_configuration(self):
         if self.configured:
@@ -361,11 +364,17 @@ class DeviceKohzuChannel(DeviceChannel):
         self.command(u"RTB{}/0\r\n".format(self.channel + 1), callback=_finish_configuration)
 
     def set_velocity(self, value):
-        value = int(value / self[ATTR_STEP_RESOLUTION][VALUE])
-        self.set_value(ATTR_START_SPEED, 2)
-        self.set_value(ATTR_TOP_SPEED, 10)
-        self.set_value(ATTR_TOP_SPEED, value)
-        self.set_value(ATTR_START_SPEED, int(value / 2.2))
+        self.set_raw_value(ATTR_VELOCITY, value)
+        value = int(ceil(value / self[ATTR_STEP_RESOLUTION][VALUE]))
+        if value == 0:
+            value = 1
+        start_speed = 500
+        acceleration = int(((value - 1000) * 4) / 1000 + 16)
+        if value < 1000:
+            start_speed = int(value / 2)
+            acceleration = 1
+        deceleration = acceleration
+        self.write_speed_table(start_speed, value, acceleration, deceleration, acceleration_pattern=u"Trapezoidal drive")
 
     def subject_update(self, key, value, subject):
         super(DeviceKohzuChannel, self).subject_update(key, value, subject)
@@ -458,8 +467,6 @@ class DeviceKohzuChannel(DeviceChannel):
                             with_token=True)
 
     def set_sys(self, setting, value, callback=None):
-        # if setting == ATTR_POSITION_UPPER_LIMIT:
-        #     print(value)
         token = self.command(
             "WSY{}/{}/{}\r\n".format(self.channel + 1, DeviceKohzu.system_setting[setting], int(value)), callback,
             with_token=True)
@@ -488,7 +495,6 @@ class DeviceKohzuChannel(DeviceChannel):
         acceleration_time = int(acceleration_time)
         deceleration_time = int(deceleration_time)
         acceleration_pattern = DeviceKohzu.accelerating_pattern[acceleration_pattern]
-
         self.command(
             "WTB{}/{}/{}/{}/{}/{}/{}\r\n".format(self.channel + 1, 0, start_speed, top_speed, acceleration_time,
                                                  deceleration_time, acceleration_pattern), callback)
