@@ -10,9 +10,9 @@ from bidict import frozenbidict, MutableBidict
 from numpy import around, sign, sqrt
 from pint.errors import UndefinedUnitError
 
+import kamzik3
 from kamzik3 import DeviceUnitError, DeviceError
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.deviceChannel import DeviceChannel
 from kamzik3.devices.devicePort import DevicePort
 from kamzik3.devices.deviceSocket import DeviceSocket
@@ -416,9 +416,9 @@ class DevicePi(DeviceSocket):
             if command == "ERR?\n" and error_code != 0:
                 readout_buffer = self.error_description.get(error_code, "Unknown error")
                 self.handle_command_error(command, readout_buffer)
-                command = RESPONSE_ERROR
+                command = RESPONSE_ERROR.decode()
             elif command not in ("#5\n", "#9\n", "STP\n") and error_code != 0:
-                command = RESPONSE_ERROR
+                command = RESPONSE_ERROR.decode()
                 readout_buffer = self.error_description.get(error_code, "Unknown error")
                 self.handle_command_error(command, readout_buffer)
 
@@ -544,10 +544,10 @@ class DevicePi(DeviceSocket):
         if callback is not None:
             self.handle_readout_callback(callback, command, readout_buffer)
         if token:
-            self.device_server.push_message(self.device_id, [command, readout_buffer], token)
+            kamzik3.session.publisher.push_message(self.device_id, [command, readout_buffer], token)
             for observer in self._observers[:]:
                 if isinstance(observer, DevicePiChannel):
-                    self.device_server.push_message(observer.device_id, [command, readout_buffer], token)
+                    kamzik3.session.publisher.push_message(observer.device_id, [command, readout_buffer], token)
 
     def start_polling(self):
         super(DevicePi, self).start_polling()
