@@ -3,7 +3,8 @@ import os
 import shutil
 import time
 from collections import OrderedDict
-
+from threading import Lock
+import oyaml as yaml
 import numpy as np
 import psutil
 
@@ -19,6 +20,7 @@ from kamzik3.snippets.snippetsControlLoops import control_asyncore_loop, control
     control_device_connection_poller
 from kamzik3.snippets.snippetsDecorators import expose_method
 from kamzik3.snippets.snippetsTimer import CallbackTimer
+from kamzik3.snippets.snippetsYaml import YamlDumper, YamlConfigSerializer
 
 
 class DeviceSession(Device):
@@ -28,6 +30,7 @@ class DeviceSession(Device):
     LOG_GUI_OUTPUT_FILE = u"gui.log"
     LOG_MACRO_OUTPUT_FILE = u"macro.log"
     log_level = logging.INFO
+    config_lock = Lock()
 
     def __init__(self, device_id=None, config=None):
         kamzik3.session = self
@@ -37,6 +40,7 @@ class DeviceSession(Device):
         self.measurement_groups = {}
         self.polling_timer = CallbackTimer(1000, self.measure_cpu_ram)
         self.os_process = psutil.Process(os.getpid())
+        self.session_config = None
         super(DeviceSession, self).__init__(device_id, config)
         self.connect()
 
@@ -79,6 +83,27 @@ class DeviceSession(Device):
         self.set_raw_value(ATTR_CPU_USAGE, self.os_process.cpu_percent())
         self.set_raw_value(ATTR_MEMORY_USAGE, self.os_process.memory_info().rss)
 
+    def set_config(self, config_file):
+        with self.config_lock:
+            with open(config_file, "r") as configFile:
+                self.session_config = (config_file, yaml.load(configFile, Loader=YamlConfigSerializer))
+
+    def update_config(self, device_id, attribute, value):
+        with self.config_lock:
+            if self.session_config is None:
+                return
+            else:
+                config_file, parsed_config = self.session_config
+                device_config = parsed_config["devices"][device_id]
+                if "config" not in device_config.conf:
+                    device_config.conf["config"] = {"attributes": {tuple(attribute): value}}
+                elif "attributes" not in device_config.conf["config"]:
+                    device_config.conf["config"]["attributes"] = {tuple(attribute): value}
+                else:
+                    device_config.conf["config"]["attributes"][tuple(attribute)] = value
+                with open(config_file, "w") as fp:
+                    yaml.dump(parsed_config, fp, Dumper=YamlDumper)
+
     def handle_configuration(self):
         start_at = time.time()
 
