@@ -416,8 +416,8 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
             elif attribute == ATTR_POSITION_UPPER_LIMIT:
                 self.set_attribute([ATTR_POSITION, MAX], value)
 
-            self.set_raw_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
-            self.set_value(attribute, value)
+            self.set_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
+            self.set_raw_value(attribute, value)
 
     @expose_method()
     def stop(self, callback=None):
