@@ -155,7 +155,7 @@ class DeviceSmarpodEcm(DeviceSocket):
                                                                                  self.error_descriptions.get(error_code,
                                                                                                              "Unknown error"))
                 self.handle_command_error(command, readout_buffer)
-                command = RESPONSE_ERROR
+                command = RESPONSE_ERROR.decode()
         elif command == u"mst?\r\n":
             if int(output) <= 1:
                 # Motor is stopped or holding
