@@ -1,3 +1,4 @@
+import hashlib
 import os
 import re
 import time
@@ -115,8 +116,13 @@ class DeviceVirtualDirectory(DeviceFileServer):
         source = drive_meta.get("source", LOCAL_FILESYSTEM)
         if source == LOCAL_FILESYSTEM:
             file_path = "/".join((input_dir, file_name))
+            with open(file_path, "rb") as file_to_check:
+                # read contents of the file
+                data = file_to_check.read()
+                # pipe contents of the file through
+                md5_hash = hashlib.md5(data).hexdigest()
             return compute_MD5("{}{}{}".format(self.device_id, drive, file_path)), os.path.getsize(
-                file_path), os.path.getmtime(file_path)
+                file_path), os.path.getmtime(file_path), md5_hash
         else:
             return source.get_meta_hash(drive=drive, input_dir=input_dir, file_name=file_name)
 
@@ -150,12 +156,16 @@ class DeviceVirtualDirectory(DeviceFileServer):
         :return: tuple (bool, path_hash)
         """
         cache_directory = kamzik3.session.get_value(ATTR_CACHE_DIRECTORY)
-        path_hash, file_size, file_mod_time = self.get_meta_hash(drive, input_dir, file_name)
+        path_hash, file_size, file_mod_time, file_md5_hash = self.get_meta_hash(drive, input_dir, file_name)
         path_hash = "/".join((cache_directory, path_hash))
         if os.path.exists(path_hash):
-            # print(file_mod_time, os.path.getmtime(path_hash), file_size, os.path.getsize(path_hash))
-            if file_size == os.path.getsize(path_hash) and file_mod_time == os.path.getmtime(path_hash):
-            # if file_mod_time == os.path.getmtime(path_hash):
+            with open(path_hash, "rb") as file_to_check:
+                # read contents of the file
+                data = file_to_check.read()
+                # pipe contents of the file through
+                cached_file_hash = hashlib.md5(data).hexdigest()
+
+            if cached_file_hash == file_md5_hash:
                 return True, path_hash
 
         return False, path_hash
