@@ -1,4 +1,5 @@
 import numpy as np
+from PyQt5.QtCore import pyqtSlot
 from PyQt5.QtWidgets import QWidget
 
 from kamzik3 import units
@@ -7,13 +8,16 @@ from kamzik3.devices.attribute import Attribute
 from kamzik3.gui.attributeDisplayWidget import AttributeDisplayWidget
 from kamzik3.snippets.snippetsYaml import YamlSerializable
 
-
 class MacroWidget(QWidget, YamlSerializable):
 
-    def __init__(self, title=None, config=None, parent=None):
+    user_mode = MODE_SIMPLE
+
+    def __init__(self, title=None, config=None, user_mode=MODE_SIMPLE, parent=None):
         self.title = title
+        self.user_mode = user_mode
         self.macro_inputs = {}
         self.macro_widgets = {}
+        self.config = config
         QWidget.__init__(self, parent)
 
     def yaml_mapping(self):
@@ -30,9 +34,6 @@ class MacroWidget(QWidget, YamlSerializable):
     def _show_method_attribute_widget(self, attribute_title, attribute, row):
         if attribute is None:
             attribute = Attribute("")
-
-        if self.config.get("defaults", False) and self.config["defaults"].get(attribute_title, None) is not None:
-            attribute[VALUE] = self.config["defaults"][attribute_title]
         attribute_widget = AttributeDisplayWidget(attribute_title, attribute, config={"label_width": 110})
         self.layout().addWidget(attribute_widget.label_widget, row, 0)
         self.layout().addWidget(attribute_widget.input_widget, row, 1)
@@ -42,8 +43,6 @@ class MacroWidget(QWidget, YamlSerializable):
 
     def _show_attribute_widget(self, row):
         attribute = self.attribute.attribute_copy()
-        if self.config.get("defaults", False):
-            attribute[VALUE] = self.config["defaults"].get("Value", 0)
         attribute_widget = AttributeDisplayWidget(u"Value", attribute, config={"label_width": 110})
         self.layout().addWidget(attribute_widget.label_widget, row, 0)
         self.layout().addWidget(attribute_widget.input_widget, row, 1)
@@ -52,7 +51,13 @@ class MacroWidget(QWidget, YamlSerializable):
         return attribute_widget
 
     def _show_start_point_widget(self, row, default_unit=None):
-        start_point_widget = AttributeDisplayWidget(u"Start point", self.attribute.attribute_copy(),
+        attribute = self.attribute.attribute_copy()
+        if default_unit is not None and attribute.value() is not None:
+            default_value = units.Quantity(self.config.get("end_point", attribute.value())).to(default_unit)
+        else:
+            default_value = self.config.get("end_point", attribute.value())
+        attribute[VALUE] = default_value
+        start_point_widget = AttributeDisplayWidget(u"Start point", attribute,
                                                     config={"label_width": 110})
         if default_unit is not None:
             start_point_widget.set_unit(default_unit)
@@ -64,8 +69,13 @@ class MacroWidget(QWidget, YamlSerializable):
         return start_point_widget
 
     def _show_end_point_widget(self, row, default_unit=None):
-        end_point_widget = AttributeDisplayWidget(u"End point", self.attribute.attribute_copy(),
-                                                  config={"label_width": 110})
+        attribute = self.attribute.attribute_copy()
+        if default_unit is not None and attribute.value() is not None:
+            default_value = units.Quantity(self.config.get("end_point", attribute.value())).to(default_unit)
+        else:
+            default_value = self.config.get("end_point", attribute.value())
+        attribute[VALUE] = default_value
+        end_point_widget = AttributeDisplayWidget(u"End point", attribute, config={"label_width": 110})
         if default_unit is not None:
             end_point_widget.set_unit(default_unit)
         self.end_point_lock_button.toggled.connect(end_point_widget.input_widget.setDisabled)
@@ -84,11 +94,6 @@ class MacroWidget(QWidget, YamlSerializable):
         if default_unit is not None:
             default_value = units.Quantity(default_value, default_unit)
         field_size_attribute[VALUE] = default_value
-        if self.config.get("defaults", False):
-            try:
-                field_size_attribute[VALUE] = self.config["defaults"].get("Field size", default_value)
-            except TypeError:
-                field_size_attribute[VALUE] = units.Quantity(self.config["defaults"].get("Field size", default_value))
         field_size_attribute[MAX] = abs(field_size_attribute[MAX] - field_size_attribute[MIN])
         field_size_attribute[MIN] = 0
         field_size_widget = AttributeDisplayWidget(u"Field size", field_size_attribute,
@@ -112,11 +117,6 @@ class MacroWidget(QWidget, YamlSerializable):
         if default_unit is not None:
             default_value = units.Quantity(default_value, default_unit)
         step_size_attribute[VALUE] = default_value
-        if self.config.get("defaults", False):
-            try:
-                step_size_attribute[VALUE] = self.config["defaults"].get("Step size", default_value)
-            except TypeError:
-                step_size_attribute[VALUE] = units.Quantity(self.config["defaults"].get("Step size", default_value))
         if self.radio_absolute_scan.isChecked():
             distance = abs(step_size_attribute[MAX] - step_size_attribute[MIN])
             step_size_attribute[MIN] = -distance
@@ -141,11 +141,10 @@ class MacroWidget(QWidget, YamlSerializable):
         return step_size_widget
 
     def _show_total_steps_widget(self, row):
-        default_value = 10
-        if self.config.get("defaults", False):
-            default_value = self.config["defaults"].get("Total steps", default_value)
+        default_value = self.config.get("total_steps", 10)
         total_steps_widget = AttributeDisplayWidget("Total steps",
-                                                    Attribute(default_value=default_value, min_value=0, default_type=np.uint64),
+                                                    Attribute(default_value=default_value, min_value=0,
+                                                              default_type=np.uint64),
                                                     config={"label_width": 110})
         self.total_steps_set_widget.layout().addWidget(total_steps_widget.input_widget)
         self.total_steps_lock_button.toggled.connect(total_steps_widget.input_widget.setDisabled)
@@ -193,8 +192,9 @@ class MacroWidget(QWidget, YamlSerializable):
         return positive_tolerance_widget
 
     def _show_random_deviation_widget(self, row):
+        default_value = units.Quantity(self.config.get("Random deviation", "0 percent")).to("percent")
         random_deviation_widget = AttributeDisplayWidget("Random deviation",
-                                                         Attribute(default_value=0, decimals=2, default_type=np.float16,
+                                                         Attribute(default_value=default_value, decimals=2, default_type=np.float16,
                                                                    min_value=0,
                                                                    max_value=100, unit=u"%"),
                                                          config={"static_unit": True, "label_width": 110})
@@ -205,8 +205,10 @@ class MacroWidget(QWidget, YamlSerializable):
         return random_deviation_widget
 
     def _show_wait_after_widget(self, row):
+        default_value = units.Quantity(self.config.get("wait_after", "0s")).to("s")
         wait_after_widget = AttributeDisplayWidget("Wait after",
-                                                   Attribute(default_value=0, decimals=0, default_type=np.float64, min_value=0,
+                                                   Attribute(default_value=default_value, decimals=0, default_type=np.float64,
+                                                             min_value=0,
                                                              unit=u"ms",
                                                              description="Set waiting time after each step."),
                                                    config={"static_unit": False, "label_width": 110})
@@ -227,11 +229,10 @@ class MacroWidget(QWidget, YamlSerializable):
         return repeat_widget
 
     def _show_retry_widget(self, row):
-        default_value = 0
-        if self.config.get("defaults", False):
-            default_value = self.config["defaults"].get("retry", default_value)
+        default_value = self.config.get("retry", 0)
         retry_widget = AttributeDisplayWidget("Retry step",
-                                              Attribute(default_value=int(default_value), default_type=np.uint32, min_value=0,
+                                              Attribute(default_value=int(default_value), default_type=np.uint32,
+                                                        min_value=0,
                                                         description="In case of error, retry_count step again.\r\nSet maximum number of retries before throwing an error."),
                                               config={"static_unit": True, "label_width": 110})
         self.layout().addWidget(retry_widget.label_widget, row, 0)
@@ -241,11 +242,10 @@ class MacroWidget(QWidget, YamlSerializable):
         return retry_widget
 
     def _show_busy_timeout(self, row):
-        default_value = 1000
-        if self.config.get("defaults", False):
-            default_value = self.config["defaults"].get("busy_timeout", default_value)
+        default_value = units.Quantity(self.config.get("busy_timeout", "1000ms")).to("ms")
         busy_timeout_widget = AttributeDisplayWidget("Busy timeout",
-                                                     Attribute(default_value=float(default_value), decimals=0, default_type=np.float64,
+                                                     Attribute(default_value=default_value, decimals=0,
+                                                               default_type=np.float64,
                                                                min_value=0,
                                                                unit=u"ms",
                                                                description="Maximum waiting time for device to change from IDLE to BUSY status. Otherwise consider step as DONE."),
@@ -256,6 +256,33 @@ class MacroWidget(QWidget, YamlSerializable):
         self.macro_inputs["busy_timeout"] = busy_timeout_widget
         return busy_timeout_widget
 
+    def _show_timeout(self, row):
+        default_value = units.Quantity(self.config.get("timeout", "0s")).to("s")
+        timeout_widget = AttributeDisplayWidget("Step timeout",
+                                                Attribute(default_value=default_value, decimals=0,
+                                                          default_type=np.float64,
+                                                          min_value=0,
+                                                          unit=u"s",
+                                                          description="Maximum waiting time for step to finish."),
+                                                config={"static_unit": False, "label_width": 110})
+        self.layout().addWidget(timeout_widget.label_widget, row, 0)
+        self.layout().addWidget(timeout_widget.input_widget, row, 1)
+        self.layout().addWidget(timeout_widget.unit_widget, row, 2)
+        self.macro_inputs["timeout"] = timeout_widget
+        return timeout_widget
+
+    def _show_on_warning_widget(self, row):
+        default_value = self.config.get("on_warning", "Abort macro")
+        on_warning_widget = AttributeDisplayWidget("On warning",
+                                              Attribute(default_value=default_value, default_type=["Abort macro", "Skip"],
+                                                        description="In case of warning, do specific action."),
+                                              config={"static_unit": True, "label_width": 110})
+        self.layout().addWidget(on_warning_widget.label_widget, row, 0)
+        self.layout().addWidget(on_warning_widget.input_widget, row, 1)
+        self.layout().addWidget(on_warning_widget.unit_widget, row, 2)
+        self.macro_inputs["on_warning"] = on_warning_widget
+        return on_warning_widget
+
     def get_input_values(self):
         output = {}
         for key, input in self.macro_inputs.items():
@@ -264,3 +291,15 @@ class MacroWidget(QWidget, YamlSerializable):
 
     def get_link(self):
         raise NotImplementedError()
+
+    @pyqtSlot()
+    def set_simple_mode(self):
+        self.user_mode = MODE_SIMPLE
+
+    @pyqtSlot()
+    def set_extended_mode(self):
+        self.user_mode = MODE_EXTENDED
+
+    @pyqtSlot()
+    def set_all_mode(self):
+        self.user_mode = MODE_ALL
\ No newline at end of file
