@@ -14,8 +14,8 @@ class MacroStepWidget(Ui_Form, MacroWidget):
     attribute = None
     debug_widget = None
 
-    def __init__(self, title=u"Step settings", config=None, parent=None):
-        MacroWidget.__init__(self, parent)
+    def __init__(self, title=u"Step settings", config=None, user_mode=MODE_SIMPLE, parent=None):
+        MacroWidget.__init__(self, user_mode=user_mode, parent=parent)
         self.config = config
         if self.config is None:
             self.config = {}
@@ -27,7 +27,7 @@ class MacroStepWidget(Ui_Form, MacroWidget):
         self.button_settings.setEnabled(False)
         self.method_attributes = []
         self.set_title(title)
-        self.spacer = self.gridLayout.itemAtPosition(7, 0)
+        self.spacer = self.gridLayout.itemAtPosition(8, 0)
         self.label_attribute.hide()
         self.combobox_attribute.hide()
         self.label_method.hide()
@@ -112,10 +112,13 @@ class MacroStepWidget(Ui_Form, MacroWidget):
 
     def _init_template_settings(self):
         if not self.config.get(CFG_TEMPLATE, False):
-            self.label_settings.hide()
-            self.widget_settings.hide()
+            if self.user_mode != MODE_ALL:
+                self.label_settings.hide()
+                self.widget_settings.hide()
             self.label_scanner.hide()
             self.combobox_scanner.hide()
+            self.label_warning.hide()
+            self.combobox_warning.hide()
 
     def _init_scanner_list(self):
         self.combobox_scanner.blockSignals(True)
@@ -177,24 +180,19 @@ class MacroStepWidget(Ui_Form, MacroWidget):
         if attribute == "":
             return
 
+        if self.user_mode == MODE_ALL:
+            self.label_settings.show()
+            self.widget_settings.show()
+            self.checkbox_log.setChecked(self.config.get("trigger_log", True))
+        else:
+            self.label_settings.hide()
+            self.widget_settings.hide()
+
         attribute = attribute.split(u" # ")
         self.attribute = self.device.get_attribute(attribute)
         row = 5
         self.macro_widgets.append(self._show_attribute_widget(row))
         row += 1
-        if self.attribute.numerical:
-            self.macro_widgets.append(self._show_negative_tolerance_widget(row))
-            row += 1
-            self.macro_widgets.append(self._show_positive_tolerance_widget(row))
-            row += 1
-            # self.macro_widgets.append(self._show_random_deviation_widget(row))
-            # row += 1
-        self.macro_widgets.append(self._show_wait_after_widget(row))
-        row += 1
-        self.macro_widgets.append(self._show_repeat_widget(row))
-        row += 1
-        self.macro_widgets.append(self._show_retry_widget(row))
-        row += 1
         if self.config.get("scanner", "None") != "None":
             scanner = kamzik3.session.get_device(self.config["scanner"])
             scanner_attributes = scanner.get_scanner_attributes()
@@ -203,7 +201,24 @@ class MacroStepWidget(Ui_Form, MacroWidget):
                     self._show_method_attribute_widget(attribute, scanner.get_attribute(attribute).attribute_copy(),
                                                        row))
                 row += 1
-        row += 1
+        if self.user_mode in (MODE_ALL, MODE_EXTENDED):
+            if self.attribute.numerical:
+                self.macro_widgets.append(self._show_negative_tolerance_widget(row))
+                row += 1
+                self.macro_widgets.append(self._show_positive_tolerance_widget(row))
+                row += 1
+                # self.macro_widgets.append(self._show_random_deviation_widget(row))
+                # row += 1
+            self.macro_widgets.append(self._show_wait_after_widget(row))
+            row += 1
+            self.macro_widgets.append(self._show_repeat_widget(row))
+            row += 1
+            self.macro_widgets.append(self._show_retry_widget(row))
+            row += 1
+            self.macro_widgets.append(self._show_timeout(row))
+            row += 1
+            self.macro_widgets.append(self._show_on_warning_widget(row))
+            row += 1
         self.layout().removeItem(self.spacer)
         self.layout().addItem(self.spacer, row, 0)
 
@@ -220,6 +235,14 @@ class MacroStepWidget(Ui_Form, MacroWidget):
         if method == "":
             return
 
+        if self.user_mode == MODE_ALL:
+            self.label_settings.show()
+            self.widget_settings.show()
+            self.checkbox_log.setChecked(self.config.get("trigger_log", True))
+        else:
+            self.label_settings.hide()
+            self.widget_settings.hide()
+
         row = 5
         for exposed_method in self.device.exposed_methods:
             if exposed_method[0] == method:
@@ -232,14 +255,6 @@ class MacroStepWidget(Ui_Form, MacroWidget):
                             self._show_method_attribute_widget(attribute_title, device_attribute, row))
                         row += 1
                 break
-        self.macro_widgets.append(self._show_wait_after_widget(row))
-        row += 1
-        self.macro_widgets.append(self._show_repeat_widget(row))
-        row += 1
-        self.macro_widgets.append(self._show_retry_widget(row))
-        row += 1
-        self.macro_widgets.append(self._show_busy_timeout(row))
-        row += 1
         if self.config.get("scanner", "None") != "None":
             scanner = kamzik3.session.get_device(self.config["scanner"])
             scanner_attributes = scanner.get_scanner_attributes()
@@ -248,6 +263,20 @@ class MacroStepWidget(Ui_Form, MacroWidget):
                     self._show_method_attribute_widget(attribute, scanner.get_attribute(attribute).attribute_copy(),
                                                        row))
                 row += 1
+        if self.user_mode in (MODE_ALL, MODE_EXTENDED):
+            self.macro_widgets.append(self._show_wait_after_widget(row))
+            row += 1
+            self.macro_widgets.append(self._show_repeat_widget(row))
+            row += 1
+            self.macro_widgets.append(self._show_retry_widget(row))
+            row += 1
+            self.macro_widgets.append(self._show_busy_timeout(row))
+            row += 1
+            self.macro_widgets.append(self._show_timeout(row))
+            row += 1
+            self.macro_widgets.append(self._show_on_warning_widget(row))
+            row += 1
+
         self.layout().removeItem(self.spacer)
         self.layout().addItem(self.spacer, row, 0)
 
@@ -278,7 +307,7 @@ class MacroStepWidget(Ui_Form, MacroWidget):
             "step": self.combobox_step.currentText(),
             "trigger_log": self.checkbox_log.isChecked(),
             "scanner": self.combobox_scanner.currentText(),
-            "defaults": self.checkbox_defaults.isChecked(),
+            "on_warning": self.combobox_warning.currentText(),
         }
         if template["step"] == "Set attribute":
             template["attribute"] = self.combobox_attribute.currentText()
@@ -299,12 +328,22 @@ class MacroStepWidget(Ui_Form, MacroWidget):
                         device_id=self.combobox_device.currentText(),
                         attribute=self.config.get("attribute").split(u" # "),
                         setpoint=values["value"],
-                        negative_tolerance=values["positive_tolerance"] if self.attribute.numerical else None,
-                        positive_tolerance=values["negative_tolerance"] if self.attribute.numerical else None,
-                        repeat_count=values["repeat"],
-                        wait_after=values["wait_after"],
-                        retry_count=int(values["retry_count"].m),
-                        trigger_log=self.config.get("trigger_log", True)
+                        negative_tolerance=self.attribute.negative_tolerance() if self.user_mode == MODE_SIMPLE else (
+                            values["negative_tolerance"] if self.attribute.numerical else None),
+                        positive_tolerance=self.attribute.positive_tolerance() if self.user_mode == MODE_SIMPLE else (
+                            values["positive_tolerance"] if self.attribute.numerical else None),
+                        repeat_count=self.config.get("repeat", 0) if self.user_mode == MODE_SIMPLE else values[
+                            "repeat"],
+                        wait_after=units.Quantity(self.config.get("wait_after", "0s") if self.user_mode == MODE_SIMPLE else
+                        values["wait_after"]).to("s"),
+                        retry_count=self.config.get("retry_count", 0) if self.user_mode == MODE_SIMPLE else int(
+                            values["retry_count"].m),
+                        trigger_log=self.config.get("trigger_log",
+                                                    True) if self.user_mode == MODE_SIMPLE else self.checkbox_log.isChecked(),
+                        on_warning=self.config.get("on_warning", "Abort macro") if self.user_mode == MODE_SIMPLE else
+                        values["on_warning"],
+                        timeout=units.Quantity(self.config.get("timeout", "0s") if self.user_mode == MODE_SIMPLE else values[
+                            "timeout"]).to("s")
                     )
                 else:
                     step = StepDeviceAttribute(
@@ -312,10 +351,18 @@ class MacroStepWidget(Ui_Form, MacroWidget):
                         device_id=self.combobox_device.currentText(),
                         attribute=self.config.get("attribute").split(u" # "),
                         setpoint=values["value"],
-                        repeat_count=values["repeat"],
-                        wait_after=values["wait_after"],
-                        retry_count=int(values["retry_count"].m),
-                        trigger_log=self.config.get("trigger_log", True)
+                        repeat_count=self.config.get("repeat", 0) if self.user_mode == MODE_SIMPLE else values[
+                            "repeat"],
+                        wait_after=units.Quantity(self.config.get("wait_after", "0s") if self.user_mode == MODE_SIMPLE else
+                        values["wait_after"]).to("s"),
+                        retry_count=self.config.get("retry_count", 0) if self.user_mode == MODE_SIMPLE else int(
+                            values["retry_count"].m),
+                        trigger_log=self.config.get("trigger_log",
+                                                    True) if self.user_mode == MODE_SIMPLE else self.checkbox_log.isChecked(),
+                        on_warning=self.config.get("on_warning", "Abort macro") if self.user_mode == MODE_SIMPLE else
+                        values["on_warning"],
+                        timeout=units.Quantity(self.config.get("timeout", "0s") if self.user_mode == MODE_SIMPLE else values[
+                            "timeout"]).to("s")
                     )
             elif self.config.get("method") is not None:
                 method_parameters = {}
@@ -330,18 +377,26 @@ class MacroStepWidget(Ui_Form, MacroWidget):
                     device_id=self.combobox_device.currentText(),
                     method=self.config.get("method"),
                     method_parameters=method_parameters,
-                    repeat_count=values["repeat"],
-                    wait_after=values["wait_after"],
-                    retry_count=int(values["retry_count"].m),
-                    busy_timeout=values["busy_timeout"],
-                    trigger_log=self.config.get("trigger_log", True)
+                    repeat_count=self.config.get("repeat", 0) if self.user_mode == MODE_SIMPLE else values["repeat"],
+                    wait_after=units.Quantity(self.config.get("wait_after", "0s") if self.user_mode == MODE_SIMPLE else values[
+                        "wait_after"]).to("s"),
+                    retry_count=self.config.get("retry_count", 0) if self.user_mode == MODE_SIMPLE else int(
+                        values["retry_count"].m),
+                    busy_timeout=units.Quantity(self.config.get("busy_timeout", "1000ms") if self.user_mode == MODE_SIMPLE else
+                    values["busy_timeout"]).to("ms"),
+                    trigger_log=self.config.get("trigger_log",
+                                                True) if self.user_mode == MODE_SIMPLE else self.checkbox_log.isChecked(),
+                    on_warning=self.config.get("on_warning", "Abort macro") if self.user_mode == MODE_SIMPLE else
+                    values["on_warning"],
+                    timeout=units.Quantity(self.config.get("timeout", "0s") if self.user_mode == MODE_SIMPLE else values[
+                        "timeout"]).to("s")
                 )
-            if self.config.get("scanner", "None") != "None":
-                scanner = kamzik3.session.get_device(self.config["scanner"])
-                scanner_attributes = {}
-                for attribute in scanner.get_scanner_attributes():
-                    scanner_attributes[attribute] = self.macro_inputs[attribute].get_attribute_value()
-                step = scanner.get_scanner_macro(step, scanner_attributes)
+            # if self.config.get("scanner", "None") != "None":
+            #     scanner = kamzik3.session.get_device(self.config["scanner"])
+            #     scanner_attributes = {}
+            #     for attribute in scanner.get_scanner_attributes():
+            #         scanner_attributes[attribute] = self.macro_inputs[attribute].get_attribute_value()
+            #     step = scanner.get_scanner_macro(scanner_input=step, scanner_attributes=scanner_attributes)
             return step
         except KeyError:
             raise KeyError("One of attributes for {} were not set".format(self.title))
@@ -360,3 +415,15 @@ class MacroStepWidget(Ui_Form, MacroWidget):
             self.debug_widget = DeviceDebugWidget(self.device)
 
         show_centralized_top(self.debug_widget)
+
+    def set_simple_mode(self):
+        MacroWidget.set_simple_mode(self)
+        self.device_selected(self.config.get("device_id"))
+
+    def set_extended_mode(self):
+        MacroWidget.set_extended_mode(self)
+        self.device_selected(self.config.get("device_id"))
+
+    def set_all_mode(self):
+        MacroWidget.set_all_mode(self)
+        self.device_selected(self.config.get("device_id"))
