@@ -1,8 +1,8 @@
 import logging
-from threading import Event, Lock, Thread
+from threading import Event, Thread
 
 import kamzik3
-from kamzik3 import DeviceError, units, SetpointTimeoutException, MacroException
+from kamzik3 import DeviceError, units, MacroException
 from kamzik3.constants import *
 from kamzik3.devices.attribute import Attribute
 from kamzik3.macro.common import Common
@@ -12,15 +12,29 @@ from kamzik3.macro.common import Common
 
 class Step(Common):
 
-    def __init__(self, common_id, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True):
+    def __init__(self, common_id, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True,
+                 on_warning=ABORT, timeout=units.Quantity(0, "s")):
         self.steps_count = 1
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step".format(common_id))
-        super(Step, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log)
+        super(Step, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout)
 
     def start(self):
         super(Step, self).start()
         self.body()
+
+        # Raise exception when step is in error state
+        if self.get_state() == WARNING:
+            if self.on_warning == ABORT or self.retry_count > 0:
+                raise MacroException(self.warning_message)
+            else:
+                # Skipping here
+                self.done()
+                return
+
+        # Raise exception when step is in error state
+        if self.get_state() == ERROR:
+            raise MacroException(self.error_message)
         if self.get_state() == STOPPED:
             return
         self.start_timer()
@@ -31,6 +45,15 @@ class Step(Common):
     def repeat(self):
         super(Step, self).repeat()
         self.body()
+        # Raise exception when step is in error state
+        if self.get_state() == WARNING:
+            if self.on_warning == ABORT or self.retry_count > 0:
+                raise MacroException(self.warning_message)
+            else:
+                return
+        # Raise exception when step is in error state
+        if self.get_state() == ERROR:
+            raise MacroException(self.error_message)
         if self.get_state() == STOPPED:
             return
         self.start_timer()
@@ -60,11 +83,12 @@ class Step(Common):
 class StepSetpoint(Step):
 
     def __init__(self, common_id, setpoint, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0,
-                 trigger_log=True):
+                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s")):
         self.setpoint = setpoint
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step".format(common_id))
-        super(StepSetpoint, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log)
+        super(StepSetpoint, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning,
+                                           timeout)
 
     def body(self):
         raise NotImplementedError()
@@ -88,17 +112,17 @@ class StepDeviceAttribute(StepSetpoint):
     device_attribute = None
 
     def __init__(self, common_id, device_id, attribute, setpoint, repeat_count=0, wait_after=units.Quantity(0, "s"),
-                 retry_count=0, trigger_log=True):
+                 retry_count=0, trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s")):
         self.device_id = device_id
         self.attribute = Attribute.list_attribute(attribute)
         self.device_status_idle_flag = False
         self.device_setpoint_reached_flag = False
         self.check_event = Event()
-        self.timeout_lock = Lock()  # Prevents race on setting check_timeout
+        # self.timeout_lock = Lock()  # Prevents race on setting check_timeout
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, attribute))
         super(StepDeviceAttribute, self).__init__(common_id, setpoint, repeat_count, wait_after, retry_count,
-                                                  trigger_log)
+                                                  trigger_log, on_warning, timeout)
 
     def body(self):
         self.check_timeout = 0
@@ -126,23 +150,19 @@ class StepDeviceAttribute(StepSetpoint):
                     self.check_event.set()
                     break
                 else:
-                    # Reset check_timeout
-                    with self.timeout_lock:
-                        self.check_timeout += 10
+                    self.check_timeout += 10
         except (DeviceError, AttributeError) as e:
             self.error(e)
-        # Raise exception when step is in error state
-        if self.get_state() == ERROR:
-            raise MacroException(self.error_message)
-        # Raise exception if setpoint was not reached
-        if self.get_state() != STOPPED and not self.check_event.isSet():
-            raise SetpointTimeoutException(
-                "{}'s attribute setpoint {}: {} could not be reached.".format(self.device.device_id, self.attribute,
-                                                                              self.setpoint))
-
         # Detach attribute and status callback
         self.device.detach_attribute_callback(self.attribute, self.check_attribute_set)
         self.device.detach_attribute_callback(ATTR_STATUS, self.check_device_status)
+
+        # Raise exception if setpoint was not reached
+        if self.get_state() != STOPPED and not self.check_event.isSet():
+            description = "{}'s attribute setpoint {}: {} could not be reached.".format(self.device.device_id,
+                                                                                        self.attribute, self.setpoint)
+            self.warning(description)
+
         # Remove device and attributes
         self.device = None
         self.device_attribute = None
@@ -158,8 +178,7 @@ class StepDeviceAttribute(StepSetpoint):
             self.device_setpoint_reached_flag = False
         # Reset check_timeout
         if not self.device_status_idle_flag:
-            with self.timeout_lock:
-                self.check_timeout = 0
+            self.check_timeout = 0
 
     def check_device_status(self, value):
         # Check if device is idle
@@ -168,8 +187,7 @@ class StepDeviceAttribute(StepSetpoint):
         else:
             self.device_status_idle_flag = False
         # Reset check_timeout
-        with self.timeout_lock:
-            self.check_timeout = 0
+        self.check_timeout = 0
 
     def on_value_set(self, key, value):
         # Check for error when setting setpoint
@@ -191,11 +209,22 @@ class StepDeviceAttribute(StepSetpoint):
 
         return lines + StepSetpoint.get_output_header(self)
 
+    def stop(self):
+        if self.device is not None and self.get_state() == BUSY:
+            self.device.stop()
+        StepSetpoint.stop(self)
+
+    def warning(self, warning_message):
+        if self.device is not None and self.get_state() == BUSY:
+            self.device.stop()
+        StepSetpoint.warning(self)
+
 
 class StepSetpointNumerical(Step):
 
     def __init__(self, common_id, setpoint, negative_tolerance=None, positive_tolerance=None, repeat_count=0,
-                 wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True):
+                 wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True, on_warning=ABORT,
+                 timeout=units.Quantity(0, "s")):
         assert isinstance(setpoint, units.Quantity)
 
         self.setpoint = setpoint
@@ -207,7 +236,8 @@ class StepSetpointNumerical(Step):
             self.positive_tolerance = units.Quantity(0, self.setpoint.u)
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step".format(common_id))
-        super(StepSetpointNumerical, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log)
+        super(StepSetpointNumerical, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log,
+                                                    on_warning, timeout)
 
     def body(self):
         raise NotImplementedError()
@@ -239,17 +269,19 @@ class StepDeviceAttributeNumerical(StepSetpointNumerical):
     device_attribute = None
 
     def __init__(self, common_id, device_id, attribute, setpoint, negative_tolerance=None, positive_tolerance=None,
-                 repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True):
+                 repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True, on_warning=ABORT,
+                 timeout=units.Quantity(0, "s")):
         self.device_id = device_id
         self.attribute = Attribute.list_attribute(attribute)
         self.device_status_idle_flag = False
         self.device_setpoint_reached_flag = False
         self.check_event = Event()
-        self.timeout_lock = Lock()  # Prevents race on setting check_timeout
+        # self.timeout_lock = Lock()  # Prevents race on setting check_timeout
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, attribute))
         super(StepDeviceAttributeNumerical, self).__init__(common_id, setpoint, negative_tolerance, positive_tolerance,
-                                                           repeat_count, wait_after, retry_count, trigger_log)
+                                                           repeat_count, wait_after, retry_count, trigger_log,
+                                                           on_warning, timeout)
 
     def body(self):
         self.check_timeout = 0
@@ -285,18 +317,14 @@ class StepDeviceAttributeNumerical(StepSetpointNumerical):
                         break
                     else:
                         # Reset check_timeout
-                        with self.timeout_lock:
-                            self.check_timeout += 10
+                        self.check_timeout += 10
         except (DeviceError, AttributeError) as e:
             self.error(e)
-        # Raise exception when step is in error state
-        if self.get_state() == ERROR:
-            raise MacroException(self.error_message)
-        # Raise exception if setpoint was not reached
+        # Raise warning if setpoint was not reached
         if self.get_state() != STOPPED and not self.check_event.isSet():
-            raise SetpointTimeoutException(
-                "{}'s attribute setpoint {}: {} could not be reached.".format(self.device.device_id, self.attribute,
-                                                                              self.setpoint))
+            description = "{}'s attribute setpoint {}: {} could not be reached.".format(self.device.device_id,
+                                                                                        self.attribute, self.setpoint)
+            self.warning(description)
 
         # Detach attribute and status callback
         self.device.detach_attribute_callback(self.attribute, self.check_attribute_set)
@@ -317,8 +345,7 @@ class StepDeviceAttributeNumerical(StepSetpointNumerical):
             self.device_setpoint_reached_flag = False
         # Reset check_timeout when device is busy
         if not self.device_status_idle_flag:
-            with self.timeout_lock:
-                self.check_timeout = 0
+            self.check_timeout = 0
 
     def check_device_status(self, value):
         # Check if device is idle
@@ -326,13 +353,10 @@ class StepDeviceAttributeNumerical(StepSetpointNumerical):
             self.device_status_idle_flag = True
         else:
             self.device_status_idle_flag = False
-        # Reset check_timeout
-        # with self.timeout_lock:
-        #     self.check_timeout = 0
 
     def on_value_set(self, key, value):
         # Check for error when setting setpoint
-        if key == RESPONSE_ERROR:
+        if key == RESPONSE_ERROR.encode():
             self.error(value)
 
     def reset(self):
@@ -351,10 +375,17 @@ class StepDeviceAttributeNumerical(StepSetpointNumerical):
         return lines + StepSetpointNumerical.get_output_header(self)
 
     def stop(self):
-        if self.device is not None and self.get_state() == BUSY:
-            self.device.stop()
+        device = kamzik3.session.get_device(self.device_id)
+        device.stop()
+        device.wait_for_status(STATUS_IDLE)
         StepSetpointNumerical.stop(self)
 
+    def warning(self, warning_message):
+        device = kamzik3.session.get_device(self.device_id)
+        device.stop()
+        device.wait_for_status(STATUS_IDLE)
+        StepSetpointNumerical.warning(self, warning_message)
+
 
 class StepDeviceMethod(Step):
     check_max_timeout = 2000
@@ -365,26 +396,24 @@ class StepDeviceMethod(Step):
 
     def __init__(self, common_id, device_id, method, method_parameters, repeat_count=0,
                  wait_after=units.Quantity(0, "s"), retry_count=0, busy_timeout=units.Quantity(1, "s"),
-                 trigger_log=True):
+                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s")):
         self.device_id = device_id
         self.method = method
         self.method_parameters = method_parameters
         self.device_status_idle_flag = False
         self.check_event = Event()
-        self.timeout_lock = Lock()  # Prevents race on setting check_timeout
+        # self.timeout_lock = Lock()  # Prevents race on setting check_timeout
         self.busy_timeout = busy_timeout
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, method))
-        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log)
+        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout)
 
     def body(self):
         self.check_timeout = 0
         try:
             self.logger.debug("Executing method {}".format(self.method))
-
             self.device = kamzik3.session.get_device(self.device_id)
             self.device_method = getattr(self.device, self.method)
-
             if not self.device.in_statuses(READY_DEVICE_STATUSES):
                 raise DeviceError(u"Device {} is not ready".format(self.device_id))
             # Attach callback for attribute and status changes
@@ -393,23 +422,15 @@ class StepDeviceMethod(Step):
             self.device_method(**self.method_parameters)
             # Check if reached setpoint or timed out
             self.device_status_idle_flag = self.device.in_statuses(IDLE_DEVICE_STATUSES)
-
-            wait_for_status_busy_timeout = self.busy_timeout.to(
-                "s").m / 0.1  # get waiting time in 100ms chunks
+            wait_for_status_busy_timeout = self.busy_timeout.to("s").m / 0.1
             while self.get_state() == BUSY and self.device_status_idle_flag and wait_for_status_busy_timeout > 0:
                 self.check_event.wait(0.1)
                 wait_for_status_busy_timeout -= 1
-
             while self.get_state() == BUSY and not self.device_status_idle_flag:
                 self.check_event.wait(0.1)
-
             self.check_event.set()
-
         except (DeviceError, AttributeError) as e:
             self.error(e)
-        # Raise exception when step is in error state
-        if self.get_state() == ERROR:
-            raise MacroException(self.error_message)
 
         # Detach attribute and status callback
         self.device.detach_attribute_callback(ATTR_STATUS, self.check_device_status)
@@ -427,14 +448,14 @@ class StepDeviceMethod(Step):
     def retry(self, retry_reason):
         try:
             # Make sure we tried to stop whatever is going on before retrying step again.
-            self.device.stop()
+            kamzik3.session.get_device(self.device_id).stop()
         except DeviceError:
             self.logger.error("Device could not be stopped")
         Step.retry(self, retry_reason)
 
     def on_value_set(self, key, value):
         # Check for error when setting setpoint
-        if key == RESPONSE_ERROR:
+        if key == RESPONSE_ERROR.encode():
             self.error(value)
 
     def reset(self):
@@ -442,9 +463,16 @@ class StepDeviceMethod(Step):
         self.check_event.clear()
         Step.reset(self)
 
+    def warning(self, warning_message):
+        device = kamzik3.session.get_device(self.device_id)
+        device.stop()
+        device.wait_for_status(STATUS_IDLE)
+        Step.warning(self, "{} executing {} on {}".format(warning_message, self.method, self.device_id))
+
     def stop(self):
-        if self.device is not None:
-            self.device.stop()
+        device = kamzik3.session.get_device(self.device_id)
+        device.stop()
+        device.wait_for_status(STATUS_IDLE)
         Step.stop(self)
 
     def get_output_header(self):
@@ -460,16 +488,17 @@ class StepDeviceMethod(Step):
         return lines + Step.get_output_header(self)
 
 
+# todo: Revise Parallel step
 class StepParallel(Step):
 
     def __init__(self, common_id, step_list, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0,
-                 trigger_log=True):
+                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s")):
         assert isinstance(step_list, list)
         self.step_list = step_list
         for step in self.step_list:
             if not isinstance(step, Step):
                 raise TypeError("Step {} is not type of Step".format(step))
-        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log)
+        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout)
 
     def body(self):
         step_threads = []
